"""
setuptools utilities for SageMath packages
"""

from __future__ import absolute_import
import os
import sys
from setuptools.command.test import test as TestCommand # for tests

class SageTest(TestCommand):
    """
    A TestCommand for running doctests with sage -t
    """
    def run_tests(self):
        errno = os.system("sage -t --force-lib .")
        if errno != 0:
            sys.exit(1)



"""
A Sphinx extension for writing SageMath-style documentation

This extension sets up:
- the styling theme
- intersphinx: enables crosslinks to the Python and SageMath documentation
- extlinks: short hand roles for wikipedia, trac, arxiv, ...
"""

sage_doc_url = "http://doc.sagemath.org/html/en/"
sage_documents = [
    "a_tour_of_sage", "constructions", "developer", "faq",
    "installation", "prep", "reference", "thematic_tutorials",
    "tutorial"
]
sage_modules = [
    "algebras", "databases", "game_theory", "logic", "monoids", "quadratic_forms",
    "semirings", "arithgroup", "data_structures", "graphs", "manifolds",
    "notebook", "quat_algebras", "stats", "asymptotic",
    "diophantine_approximation", "groups", "matrices", "number_fields",
    "quivers", "structure", "calculus", "discrete_geometry", "hecke", "matroids",
    "numerical", "references", "tensor", "categories", "doctest",
    "history_and_license", "misc", "padics", "repl", "tensor_free_modules",
    "coding", "dynamics", "homology", "modabvar", "parallel",
    "riemannian_geometry", "coercion", "finance", "hyperbolic_geometry",
    "modfrm", "plot3d", "rings", "combinat", "finite_rings", "interfaces",
    "modfrm_hecketriangle", "plotting", "rings_numerical", "constants",
    "function_fields", "knots", "modmisc", "polynomial_rings", "rings_standard",
    "cryptography", "functions", "lfunctions", "modsym", "power_series", "sat",
    "curves", "games", "libs", "modules", "probability", "schemes",
]

import os
import sys
pythonversion = sys.version.split(' ')[0]

def setup(app):
    """
    Initialize this Sphinx extension
    """
    app.setup_extension('sphinx.ext.todo')
    app.setup_extension('sphinx.ext.mathjax')

    app.setup_extension("sphinx.ext.intersphinx")
    app.config.intersphinx_mapping.update({
        'https://docs.python.org/': None
        })
    app.config.intersphinx_mapping.update({
        sage_doc_url + doc + "/": None
        for doc in sage_documents
        })
    app.config.intersphinx_mapping.update({
        sage_doc_url + "reference/" + module: None
        for module in sage_modules
        })

    app.setup_extension("sphinx.ext.extlinks")
    app.config.extlinks.update({
        'python': ('https://docs.python.org/release/'+pythonversion+'/%s', ''),
        # Sage trac ticket shortcuts. For example, :trac:`7549` .
        'trac': ('https://trac.sagemath.org/%s', 'trac ticket #'),
        'wikipedia': ('https://en.wikipedia.org/wiki/%s', 'Wikipedia article '),
        'arxiv': ('http://arxiv.org/abs/%s', 'Arxiv '),
        'oeis': ('https://oeis.org/%s', 'OEIS sequence '),
        'doi': ('https://dx.doi.org/%s', 'doi:'),
        'pari': ('http://pari.math.u-bordeaux.fr/dochtml/help/%s', 'pari:'),
        'mathscinet': ('http://www.ams.org/mathscinet-getitem?mr=%s', 'MathSciNet ')
        })

    app.config.html_theme = 'sage'

def themes_path():
    """
    Retrieve the location of the themes directory from the location of this package

    This is taken from Sphinx's theme documentation
    """
    package_dir = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(package_dir, 'themes')



## -*- encoding: utf-8 -*-

import os
from setuptools import setup, find_packages
from codecs import open # To open the README file with proper encoding

# Adapted from https://stackoverflow.com/questions/27664504/how-to-add-package-data-recursively-in-python-setup-py
def package_files(directory):
    return [os.path.join('..', path, filename)
                for (path, directories, filenames) in os.walk(directory)
                for filename in filenames]

setup(
    name = "sage-package",
    version = "0.0.2",
    description = 'Utilities for authoring SageMath packages',
    # get the long description from the README
    long_description = open("README.rst", encoding='utf-8').read(),
    url = 'https://github.com/sagemath/sage-package',
    author = 'Nicolas M. Thiéry et al.',
    author_email = 'nthiery@users.sf.net',
    license = 'GPLv2+',
    classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Science/Research',
      'Topic :: Software Development :: Build Tools',
      'Topic :: Scientific/Engineering :: Mathematics',
      'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
      'Programming Language :: Python :: 2.7',
    ],
    keywords = "SageMath, Sphinx, packaging",
    packages = find_packages(),
    package_data = { 'sage_package': package_files("sage_package/themes") },
    entry_points = {
        'sphinx_themes': ['path = sage_package.sphinx:themes_path']
        },
)
